@@ -3,16 +3,20 @@ package GD;
 # Copyright 1995 Lincoln D. Stein.  See accompanying README file for
 # usage information
 
+use strict;
 require 5.004;
 require FileHandle;
 require Exporter;
 require DynaLoader;
 require AutoLoader;
-use Symbol 'gensym','qualify_to_ref';
 use Carp 'croak','carp';
-use strict;
+
+use GD::Image;
+use GD::Polygon;
+
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $AUTOLOAD);
-$VERSION = "2.19";
+
+$VERSION = 2.27;
 
 @ISA = qw(Exporter DynaLoader);
 # Items to export into callers namespace by default. Note: do not export
@@ -69,7 +73,6 @@ $VERSION = "2.19";
 
 # documentation error
 *GD::Polygon::delete = \&GD::Polygon::deletePt;
-*GD::Image::stringTTF = \&GD::Image::stringFT;
 
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
@@ -78,7 +81,7 @@ sub AUTOLOAD {
 
     my($constname);
     ($constname = $AUTOLOAD) =~ s/.*:://;
-    my $val = constant($constname, @_ ? $_[0] : 0);
+    my $val = constant($constname);
     if ($! != 0) {
 	if ($! =~ /Invalid/) {
 	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
@@ -98,324 +101,25 @@ bootstrap GD;
 
 # Preloaded methods go here.
 sub GD::gdSmallFont {
-    return &GD::Font::Small;
+    return GD::Font->Small;
 }
 
 sub GD::gdLargeFont {
-    return &GD::Font::Large;
+    return GD::Font->Large;
 }
 
 sub GD::gdMediumBoldFont {
-    return &GD::Font::MediumBold;
+    return GD::Font->MediumBold;
 }
 
 sub GD::gdTinyFont {
-    return &GD::Font::Tiny;
+    return GD::Font->Tiny;
 }
 
 sub GD::gdGiantFont {
-    return &GD::Font::Giant;
+    return GD::Font->Giant;
 }
 
-
-sub GD::Image::_make_filehandle {
-  shift;  # get rid of class
-  no strict 'refs';
-  my $thing = shift;
-  return $thing if defined(fileno $thing);
-
-  # otherwise try qualifying it into caller's package
-  my $fh;
-  {   
-    local $^W = 0;  # to avoid uninitialized variable warning from Symbol.pm
-    $fh = qualify_to_ref($thing,caller(2));
-  }
-  return $fh if defined(fileno $fh);
-
-  # otherwise treat it as a file to open
-  $fh = gensym;
-  open($fh,$thing) || return;
-  return $fh;
-}
-
-sub GD::Image::new {
-  my $pack = shift;
-  if (@_ == 1) {
-    if (my $type = _image_type($_[0])) {
-      my $method = "newFrom${type}Data";
-      return unless $pack->can($method);
-      return $pack->$method($_[0]);
-    }
-    return unless my $fh = $pack->_make_filehandle($_[0]);
-    my $magic;
-    return unless read($fh,$magic,4);
-    return unless my $type = _image_type($magic);
-    seek($fh,0,0);
-    my $method = "newFrom${type}";
-    return $pack->$method($fh);
-  }
-  return $pack->_new(@_);
-}
-
-sub GD::Image::newTrueColor {
-  my $pack = shift;
-  return $pack->_new(@_, 1);
-}
-
-sub GD::Image::newPalette {
-  my $pack = shift;
-  return $pack->_new(@_, 0);
-}
-
-sub GD::Image::newFromPng {
-    croak("Usage: newFromPng(class,filehandle,[truecolor])") unless @_>=2;
-    my($class) = shift;
-    my($f)     = shift;
-    my $fh = $class->_make_filehandle($f);
-    binmode($fh);
-    $class->_newFromPng($fh,@_);
-}
-
-sub GD::Image::newFromJpeg {
-    croak("Usage: newFromJpeg(class,filehandle,[truecolor])") unless @_>=2;
-    my($class) = shift;
-    my($f)     = shift;
-    my $fh = $class->_make_filehandle($f);
-    binmode($fh);
-    $class->_newFromJpeg($fh,@_);
-}
-
-sub GD::Image::newFromGif {
-    croak("Usage: newFromGif(class,filehandle,[truecolor])") unless @_>=2;
-    my($class) = shift;
-    my($f)     = shift;
-    my $fh = $class->_make_filehandle($f);
-    binmode($fh);
-    $class->_newFromGif($fh,@_);
-}
-
-sub GD::Image::newFromXbm {
-    croak("Usage: newFromXbm(class,filehandle)") unless @_==2;
-    my($class,$f) = @_;
-    my $fh = $class->_make_filehandle($f);
-    binmode($fh);
-    $class->_newFromXbm($fh);
-}
-
-sub GD::Image::newFromGd {
-    croak("Usage: newFromGd(class,filehandle)") unless @_==2;
-    my($class,$f) = @_;
-    my $fh = $class->_make_filehandle($f);
-    binmode($fh);
-    $class->_newFromGd($fh);
-}
-
-sub GD::Image::newFromGd2 {
-    croak("Usage: newFromGd2(class,filehandle)") unless @_==2;
-    my($class,$f) = @_;
-    my $fh = $class->_make_filehandle($f);
-    binmode($fh);
-    $class->_newFromGd2($fh);
-}
-
-sub GD::Image::newFromGd2Part {
-    croak("Usage: newFromGd2(class,filehandle,srcX,srcY,width,height)") unless @_==6;
-    my($class,$f) = splice(@_,0,2);
-    my $fh = $class->_make_filehandle($f);
-    binmode($fh);
-    $class->_newFromGd2Part($fh,@_);
-}
-
-sub GD::Image::ellipse ($$$$$) {
-  my ($self,$cx,$cy,$width,$height,$color) = @_;
-  $self->arc($cx,$cy,$width,$height,0,360,$color);
-}
-
-sub GD::Image::width {
-  my $self = shift;
-  my @bounds = $self->getBounds;
-  $bounds[0];
-}
-
-sub GD::Image::height {
-  my $self = shift;
-  my @bounds = $self->getBounds;
-  $bounds[1];
-}
-
-sub _image_type {
-  my $data = shift;
-  my $magic = substr($data,0,4);
-  return 'Png'  if $magic eq "\x89PNG";
-  return 'Jpeg' if $magic eq "\377\330\377\340";
-  return 'Jpeg' if $magic eq "\377\330\377\341";
-  return 'Jpeg' if $magic eq "\377\330\377\356";
-  return 'Gif'  if $magic eq "GIF8";
-  return 'Gd2'  if $magic eq "gd2\000";
-  return 'Xpm'  if substr($data,0,9) eq "/* XPM */";
-  return;
-}
-
-
-sub GD::Image::clone {
-  croak("Usage: clone(\$image)") unless @_ == 1;
-  my $self = shift;
-  my ($x,$y) = $self->getBounds;
-  my $new = $self->new($x,$y);
-  return unless $new;
-  $new->copy($self,0,0,0,0,$x,$y);
-  return $new;
-}
-
-### The polygon object ###
-# create a new polygon
-sub GD::Polygon::new {
-    my $class = shift;
-    return bless { 'length'=>0,'points'=>[] },$class;
-}
-
-# automatic destruction of the polygon
-sub GD::Polygon::DESTROY {
-    my $self = shift;
-    undef $self->{'points'};
-}
-
-# add an x,y vertex to the polygon
-sub GD::Polygon::addPt {
-    my($self,$x,$y) = @_;
-    push(@{$self->{'points'}},[$x,$y]);
-    $self->{'length'}++;
-}
-
-# get a vertex
-sub GD::Polygon::getPt {
-    my($self,$index) = @_;
-    return () unless ($index>=0) && ($index<$self->{'length'});
-    return @{$self->{'points'}->[$index]};
-}
-
-# change the value of a vertex
-sub GD::Polygon::setPt {
-    my($self,$index,$x,$y) = @_;
-    unless (($index>=0) && ($index<$self->{'length'})) {
-	carp "Attempt to set an undefined polygon vertex";
-	return undef;
-    }
-    @{$self->{'points'}->[$index]} = ($x,$y);
-    1;
-}
-
-# return the total number of vertices
-sub GD::Polygon::length {
-    my $self = shift;
-    return $self->{'length'};
-}
-
-# return the array of vertices.
-# each vertex is an two-member (x,y) array
-sub GD::Polygon::vertices {
-    my $self = shift;
-    return @{$self->{'points'}};
-}
-
-# return the bounding box of the polygon
-# (smallest rectangle that contains it)
-sub GD::Polygon::bounds {
-    my $self = shift;
-    my($top,$bottom,$left,$right) = @_;
-    $top =    99999999;
-    $bottom =-99999999;
-    $left =   99999999;
-    $right = -99999999;
-    my $v;
-    foreach $v ($self->vertices) {
-	$left = $v->[0] if $left > $v->[0];
-	$right = $v->[0] if $right < $v->[0];
-	$top = $v->[1] if $top > $v->[1];
-	$bottom = $v->[1] if $bottom < $v->[1];
-    }
-    return ($left,$top,$right,$bottom);
-}
-
-# delete a vertex, returning it, just for fun
-sub GD::Polygon::deletePt {
-     my($self,$index) = @_;
-     unless (($index>=0) && ($index<@{$self->{'points'}})) {
- 	carp "Attempt to delete an undefined polygon vertex";
- 	return undef;
-     }
-      my($vertex) = splice(@{$self->{'points'}},$index,1);
-     $self->{'length'}--;
-      return @$vertex;
-  }
-
-# translate the polygon in space by deltaX and deltaY
-sub GD::Polygon::offset {
-    my($self,$dh,$dv) = @_;
-    my $size = $self->length;
-    my($i);
-    for ($i=0;$i<$size;$i++) {
-	my($x,$y)=$self->getPt($i);
-	$self->setPt($i,$x+$dh,$y+$dv);
-    }
-}
-
-# map the polygon from sourceRect to destRect,
-# translating and resizing it if necessary
-sub GD::Polygon::map {
-    my($self,$srcL,$srcT,$srcR,$srcB,$destL,$destT,$destR,$destB) = @_;
-    my($factorV) = ($destB-$destT)/($srcB-$srcT);
-    my($factorH) = ($destR-$destL)/($srcR-$srcL);
-    my($vertices) = $self->length;
-    my($i);
-    for ($i=0;$i<$vertices;$i++) {
-	my($x,$y) = $self->getPt($i);
-	$x = int($destL + ($x - $srcL) * $factorH);
-	$y = int($destT + ($y - $srcT) * $factorV);
-	$self->setPt($i,$x,$y);
-    }
-}
-
-# draws closed polygon with the specified color
-sub GD::Image::polygon {
-    my $self = shift;
-    my($p,$c) = @_;
-    $self->openPolygon($p, $c);
-    $self->line( @{$p->{'points'}->[0]},
-	    @{$p->{'points'}->[$p->{'length'}-1]}, $c);
-}
-
-# These routines added by Winfriend Koenig.
-sub GD::Polygon::toPt {
-    my($self, $dx, $dy) = @_;
-    unless ($self->length > 0) {
-	$self->addPt($dx,$dy);
-	return;
-    }
-    my ($x, $y) = $self->getPt($self->length-1);
-    $self->addPt($x+$dx,$y+$dy);
-}
-
-sub GD::Polygon::transform($$$$$$$) {
-    # see PostScript Ref. page 154
-    my($self, $a, $b, $c, $d, $tx, $ty) = @_;
-    my $size = $self->length;
-    for (my $i=0;$i<$size;$i++) {
-	my($x,$y)=$self->getPt($i);
-	$self->setPt($i, $a*$x+$c*$y+$tx, $b*$x+$d*$y+$ty);
-    }
-}
-
-sub GD::Polygon::scale {
-    my($self, $sx, $sy) = @_;
-    $self->transform($sx,0,0,$sy,0,0);
-}
-
-
-# Autoload methods go after __END__, and are processed by the autosplit program.
-1;
-__END__
-
 =head1 NAME
 
 GD.pm - Interface to Gd Graphics Library
@@ -625,9 +329,6 @@ the thing at the other end of the filehandle is not a valid PNG file,
 the call returns undef.  Notice that the call doesn't automatically
 close the filehandle for you.  But it does call C<binmode(FILEHANDLE)>
 for you, on platforms where this matters.
-The optional $truecolor (0/1) value can be used to override the global
-setting of trueColor() to specify if the return image should be
-palette-based or truecolor.
 
 You may use any of the following as the argument:
 
@@ -650,10 +351,13 @@ and read the PNG information from it.
   $myImage = newFromPng GD::Image('barnswallow.png');
 
 To get information about the size and color usage of the information,
-you can call the image query methods described below.
+you can call the image query methods described below. Images created
+by reading PNG images will be truecolor if the image file itself is
+truecolor. To force the image to be palette-based, pass a value of 0
+in the optional $truecolor argument.
 
 The newFromPngData() method will create a new GD::Image initialized
-with the PNG format B<data> contained in C<$data>.
+with the PNG format B<data> contained in C<$data>. 
 
 =item B<$image = GD::Image-E<gt>newFromJpeg($file, [$truecolor])>
 
@@ -662,23 +366,22 @@ with the PNG format B<data> contained in C<$data>.
 These methods will create an image from a JPEG file.  They work just
 like newFromPng() and newFromPngData(), and will accept the same
 filehandle and pathname arguments.
-The optional $truecolor (0/1) value can be used to override the global
-setting of trueColor() to specify if the return image should be
-palette-based or truecolor.
 
-=item B<$image = GD::Image-E<gt>newFromGif($file, [$truecolor])>
+Images created by reading JPEG images will always be truecolor.  To
+force the image to be palette-based, pass a value of 0 in the optional
+$truecolor argument.
 
-=item B<$image = GD::Image-E<gt>newFromGif($data, [$truecolor])>
+=item B<$image = GD::Image-E<gt>newFromGif($file)>
+
+=item B<$image = GD::Image-E<gt>newFromGif($data)>
 
 These methods will create an image from a GIF file.  They work just
 like newFromPng() and newFromPngData(), and will accept the same
 filehandle and pathname arguments.
-The optional $truecolor (0/1) value can be used to override the global
-setting of trueColor() to specify if the return image should be
-palette-based or truecolor.
 
-Bear in mind that GIF is an 8-bit format, so there will initially be
-at most 256 colors even if you specify truecolor. 
+Images created from GIFs are always 8-bit palette images. To convert
+to truecolor, you must create a truecolor image and then perform a
+copy.
 
 =item B<$image = GD::Image-E<gt>newFromXbm($file)>
 
@@ -1169,6 +872,11 @@ Example:
 
 =item B<$image-E<gt>dashedLine($x1,$y1,$x2,$y2,$color)>
 
+DEPRECATED: The libgd library provides this method solely for backward
+compatibility with libgd version 1.0, and there have been reports that
+it no longer works as expected. Please use the setStyle() and gdStyled
+methods as described below.
+
 This draws a dashed line from (x1,y1) to (x2,y2) in the specified
 color.  A more powerful way to generate arbitrary dashed and dotted
 lines is to use the setStyle() method described below and to draw with
@@ -1448,6 +1156,15 @@ pixels of the source area rather than selecting one representative
 pixel. This method is identical to copyResized() when the destination
 image is a palette image.
 
+=item B<$image-E<gt>copyRotated($sourceImage,$dstX,$dstY,>
+
+B<				$srcX,$srcY,$width,$height,$angle)>
+
+Like copyResized() but the $angle argument specifies an arbitrary
+amount to rotate the image clockwise (in degrees).  In addition, $dstX
+and $dstY species the B<center> of the destination image, and not the
+topleft corner.
+
 =item B<$image-E<gt>trueColorToPalette([$dither], [$colors])>
 
 This method converts a truecolor image to a palette image. The code for
@@ -1693,7 +1410,7 @@ with 0x7f(rr)(gg)(bb) will be transparent. The Alpha value must be >=
 
 =over 4
 
-=item B<$image-E<gt>alphaBlending($blending)>
+=item B<$image-E<gt>alphaBlending($integer)>
 
 The alphaBlending() method allows for two different modes of drawing
 on truecolor images. In blending mode, which is on by default (libgd
@@ -1706,6 +1423,8 @@ opaque. In non-blending mode, the drawing color is copied literally
 with its alpha channel information, replacing the destination
 pixel. Blending mode is not available when drawing on palette images.
 
+Pass a value of 1 for blending mode, and 0 for non-blending mode.
+
 =item B<$image-E<gt>saveAlpha($saveAlpha)>
 
 By default, GD (libgd 2.0.2 and above) does not attempt to save full
@@ -2026,3 +1745,7 @@ L<GD::Simple>,
 L<Image::Magick>
 
 =cut
+
+1;
+
+__END__
